@@ -518,6 +518,11 @@ impl<'g> TestList<'g> {
         self.rust_suites.iter()
     }
 
+    /// Looks up a test suite by binary ID.
+    pub fn get_suite(&self, binary_id: &RustBinaryId) -> Option<&RustTestSuite<'_>> {
+        self.rust_suites.get(binary_id)
+    }
+
     /// Iterates over the list of tests, returning the path and test name.
     pub fn iter_tests(&self) -> impl Iterator<Item = TestInstance<'_>> + '_ {
         self.rust_suites.iter().flat_map(|test_suite| {
@@ -1135,30 +1140,12 @@ impl<'a> TestInstance<'a> {
         &self,
         ctx: &TestExecuteContext<'_>,
         test_list: &TestList<'_>,
-        wrapper_script: Option<&'a WrapperScriptConfig>,
+        wrapper_script: Option<&WrapperScriptConfig>,
         extra_args: &[String],
         interceptor: &Interceptor,
     ) -> TestCommand {
         // TODO: non-rust tests
-
-        let platform_runner = ctx
-            .target_runner
-            .for_build_platform(self.suite_info.build_platform);
-
-        let mut cli = TestCommandCli::default();
-        cli.apply_wrappers(
-            wrapper_script,
-            platform_runner,
-            test_list.workspace_root(),
-            &test_list.rust_build_meta().target_directory,
-        );
-        cli.push(self.suite_info.binary_path.as_str());
-
-        cli.extend(["--exact", self.name, "--nocapture"]);
-        if self.test_info.ignored {
-            cli.push("--ignored");
-        }
-        cli.extend(extra_args.iter().map(String::as_str));
+        let cli = self.compute_cli(ctx, test_list, wrapper_script, extra_args);
 
         let lctx = LocalExecuteContext {
             phase: TestCommandPhase::Run,
@@ -1182,6 +1169,46 @@ impl<'a> TestInstance<'a> {
             interceptor,
         )
     }
+
+    pub(crate) fn command_line(
+        &self,
+        ctx: &TestExecuteContext<'_>,
+        test_list: &TestList<'_>,
+        wrapper_script: Option<&WrapperScriptConfig>,
+        extra_args: &[String],
+    ) -> Vec<String> {
+        self.compute_cli(ctx, test_list, wrapper_script, extra_args)
+            .to_owned_cli()
+    }
+
+    fn compute_cli(
+        &self,
+        ctx: &'a TestExecuteContext<'_>,
+        test_list: &TestList<'_>,
+        wrapper_script: Option<&'a WrapperScriptConfig>,
+        extra_args: &'a [String],
+    ) -> TestCommandCli<'a> {
+        let platform_runner = ctx
+            .target_runner
+            .for_build_platform(self.suite_info.build_platform);
+
+        let mut cli = TestCommandCli::default();
+        cli.apply_wrappers(
+            wrapper_script,
+            platform_runner,
+            test_list.workspace_root(),
+            &test_list.rust_build_meta().target_directory,
+        );
+        cli.push(self.suite_info.binary_path.as_str());
+
+        cli.extend(["--exact", self.name, "--nocapture"]);
+        if self.test_info.ignored {
+            cli.push("--ignored");
+        }
+        cli.extend(extra_args.iter().map(String::as_str));
+
+        cli
+    }
 }
 
 #[derive(Clone, Debug, Default)]
