@@ -8,8 +8,11 @@
 
 use super::{FinalStatusLevel, StatusLevel, TestOutputDisplay};
 use crate::{
-    config::{elements::LeakTimeoutResult, scripts::ScriptId},
-    list::{TestInstance, TestInstanceId, TestList},
+    config::{
+        elements::{LeakTimeoutResult, SlowTimeoutResult},
+        scripts::ScriptId,
+    },
+    list::{TestInstanceId, TestList},
     runner::{StressCondition, StressCount},
     test_output::ChildExecutionOutput,
 };
@@ -161,29 +164,16 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that was started.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// Current run statistics so far.
         current_stats: RunStats,
 
         /// The number of tests currently running, including this one.
         running: usize,
-    },
 
-    /// Show test-specific progress.
-    ///
-    /// This event is emitted shortly after a test starts running, and is used
-    /// as a trigger to show test-related progress. This is to prevent many very
-    /// short-lived tests from overwhelming the display code.
-    TestShowProgress {
-        /// If a stress test is being run, the stress index, starting from 0.
-        stress_index: Option<StressIndex>,
-
-        /// Retry data.
-        retry_data: RetryData,
-
-        /// The test instance that was started.
-        test_instance: TestInstance<'a>,
+        /// The command line that will be used to run this test.
+        command_line: Vec<String>,
     },
 
     /// A test was slower than a configured soft timeout.
@@ -192,7 +182,7 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that was slow.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// Retry data.
         retry_data: RetryData,
@@ -212,7 +202,7 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that is being retried.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// The status of this attempt to run the test. Will never be success.
         run_status: ExecuteStatus,
@@ -222,6 +212,9 @@ pub enum TestEventKind<'a> {
 
         /// Whether failure outputs are printed out.
         failure_output: TestOutputDisplay,
+
+        /// The current number of running tests.
+        running: usize,
     },
 
     /// A retry has started.
@@ -230,10 +223,16 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that is being retried.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// Data related to retries.
         retry_data: RetryData,
+
+        /// The current number of running tests.
+        running: usize,
+
+        /// The command line that will be used to run this test.
+        command_line: Vec<String>,
     },
 
     /// A test finished running.
@@ -242,7 +241,7 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that finished running.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// Test setting for success output.
         success_output: TestOutputDisplay,
@@ -272,7 +271,7 @@ pub enum TestEventKind<'a> {
         stress_index: Option<StressIndex>,
 
         /// The test instance that was skipped.
-        test_instance: TestInstance<'a>,
+        test_instance: TestInstanceId<'a>,
 
         /// The reason this test was skipped.
         reason: MismatchReason,
@@ -529,23 +528,26 @@ pub struct RunStats {
     /// The number of setup scripts that timed out.
     pub setup_scripts_timed_out: usize,
 
-    /// The number of tests that passed. Includes `passed_slow`, `flaky` and `leaky`.
+    /// The number of tests that passed. Includes `passed_slow`, `passed_timed_out`, `flaky` and `leaky`.
     pub passed: usize,
 
     /// The number of slow tests that passed.
     pub passed_slow: usize,
 
+    /// The number of timed out tests that passed.
+    pub passed_timed_out: usize,
+
     /// The number of tests that passed on retry.
     pub flaky: usize,
 
-    /// The number of tests that failed.
+    /// The number of tests that failed. Includes `leaky_failed`.
     pub failed: usize,
 
     /// The number of failed tests that were slow.
     pub failed_slow: usize,
 
-    /// The number of tests that timed out.
-    pub timed_out: usize,
+    /// The number of timed out tests that failed.
+    pub failed_timed_out: usize,
 
     /// The number of tests that passed but leaked handles.
     pub leaky: usize,
@@ -577,7 +579,7 @@ impl RunStats {
 
     /// Returns count of tests that did not pass.
     pub fn failed_count(&self) -> usize {
-        self.failed + self.exec_failed + self.timed_out
+        self.failed + self.exec_failed + self.failed_timed_out
     }
 
     /// Summarizes the stats as an enum at the end of a test run.
@@ -650,7 +652,8 @@ impl RunStats {
             ExecutionResult::ExecFail => {
                 self.setup_scripts_exec_failed += 1;
             }
-            ExecutionResult::Timeout => {
+            // Timed out setup scripts are always treated as failures
+            ExecutionResult::Timeout { .. } => {
                 self.setup_scripts_timed_out += 1;
             }
         }
@@ -704,7 +707,20 @@ impl RunStats {
                     self.failed_slow += 1;
                 }
             }
-            ExecutionResult::Timeout => self.timed_out += 1,
+            ExecutionResult::Timeout {
+                result: SlowTimeoutResult::Pass,
+            } => {
+                self.passed += 1;
+                self.passed_timed_out += 1;
+                if run_statuses.len() > 1 {
+                    self.flaky += 1;
+                }
+            }
+            ExecutionResult::Timeout {
+                result: SlowTimeoutResult::Fail,
+            } => {
+                self.failed_timed_out += 1;
+            }
             ExecutionResult::ExecFail => self.exec_failed += 1,
         }
     }
@@ -1048,7 +1064,10 @@ pub enum ExecutionResult {
     /// An error occurred while executing the test.
     ExecFail,
     /// The test was terminated due to a timeout.
-    Timeout,
+    Timeout {
+        /// Whether this timeout was treated as a failure.
+        result: SlowTimeoutResult,
+    },
 }
 
 impl ExecutionResult {
@@ -1056,6 +1075,9 @@ impl ExecutionResult {
     pub fn is_success(self) -> bool {
         match self {
             ExecutionResult::Pass
+            | ExecutionResult::Timeout {
+                result: SlowTimeoutResult::Pass,
+            }
             | ExecutionResult::Leak {
                 result: LeakTimeoutResult::Pass,
             } => true,
@@ -1064,7 +1086,34 @@ impl ExecutionResult {
             }
             | ExecutionResult::Fail { .. }
             | ExecutionResult::ExecFail
-            | ExecutionResult::Timeout => false,
+            | ExecutionResult::Timeout {
+                result: SlowTimeoutResult::Fail,
+            } => false,
+        }
+    }
+
+    /// Returns true if this result represents a test that was terminated by nextest
+    /// (as opposed to failing naturally).
+    ///
+    /// This is used to suppress output spam when immediate termination is active.
+    ///
+    /// TODO: This is a heuristic that checks if the test was terminated by SIGTERM (Unix) or
+    /// job object (Windows). In an edge case, a test could send SIGTERM to itself, which would
+    /// incorrectly be detected as a nextest-initiated termination. A more robust solution would
+    /// track which tests were explicitly sent termination signals by nextest.
+    pub fn is_termination_failure(&self) -> bool {
+        match self {
+            #[cfg(unix)]
+            ExecutionResult::Fail {
+                failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(libc::SIGTERM)),
+                ..
+            } => true,
+            #[cfg(windows)]
+            ExecutionResult::Fail {
+                failure_status: FailureStatus::Abort(AbortStatus::JobObject),
+                ..
+            } => true,
+            _ => false,
         }
     }
 
@@ -1075,7 +1124,7 @@ impl ExecutionResult {
             ExecutionResult::Leak { .. } => "leak",
             ExecutionResult::Fail { .. } => "fail",
             ExecutionResult::ExecFail => "exec-fail",
-            ExecutionResult::Timeout => "timeout",
+            ExecutionResult::Timeout { .. } => "timeout",
         }
     }
 }
@@ -1172,6 +1221,9 @@ pub enum CancelReason {
     /// The global timeout was exceeded.
     GlobalTimeout,
 
+    /// A test failed and fail-fast with immediate termination was specified.
+    TestFailureImmediate,
+
     /// A termination signal (on Unix, SIGTERM or SIGHUP) was received.
     Signal,
 
@@ -1189,6 +1241,7 @@ impl CancelReason {
             CancelReason::TestFailure => "test failure",
             CancelReason::ReportError => "reporting error",
             CancelReason::GlobalTimeout => "global timeout",
+            CancelReason::TestFailureImmediate => "test failure",
             CancelReason::Signal => "signal",
             CancelReason::Interrupt => "interrupt",
             CancelReason::SecondSignal => "second signal",
@@ -1555,7 +1608,19 @@ mod tests {
             RunStats {
                 initial_run_count: 42,
                 finished_count: 42,
-                timed_out: 1,
+                failed_timed_out: 1,
+                ..RunStats::default()
+            }
+            .summarize_final(),
+            FinalRunStats::Failed(RunStatsFailureKind::Test {
+                initial_run_count: 42,
+                not_run: 0
+            }),
+            "timed out => failure {:?} {:?}",
+            RunStats {
+                initial_run_count: 42,
+                finished_count: 42,
+                failed_timed_out: 1,
                 ..RunStats::default()
             }
             .summarize_final(),
@@ -1563,7 +1628,6 @@ mod tests {
                 initial_run_count: 42,
                 not_run: 0
             }),
-            "timed out => failure"
         );
         assert_eq!(
             RunStats {
