@@ -94,6 +94,14 @@ impl<'a> ExecutorContext<'a> {
         }
     }
 
+    fn test_execute_context(&self) -> TestExecuteContext<'_> {
+        TestExecuteContext {
+            profile_name: self.profile.name(),
+            double_spawn: &self.double_spawn,
+            target_runner: &self.target_runner,
+        }
+    }
+
     /// Run scripts, returning data about each successfully executed script.
     pub(super) async fn run_setup_scripts(
         &self,
@@ -210,11 +218,20 @@ impl<'a> ExecutorContext<'a> {
 
         let (req_rx_tx, req_rx_rx) = oneshot::channel();
 
+        let ctx = self.test_execute_context();
+        let command_line = test.instance.command_line(
+            &ctx,
+            self.test_list,
+            settings.run_wrapper(),
+            settings.run_extra_args(),
+        );
+
         // Wait for the Started event to be processed by the
         // execution future.
         _ = resp_tx.send(ExecutorEvent::Started {
             stress_index,
             test_instance: test.instance,
+            command_line: command_line.clone(),
             req_rx_tx,
         });
         let mut req_rx = match req_rx_rx.await {
@@ -245,6 +262,7 @@ impl<'a> ExecutorContext<'a> {
                     stress_index,
                     test_instance: test.instance,
                     retry_data,
+                    command_line: command_line.clone(),
                     tx,
                 });
 
@@ -666,11 +684,7 @@ impl<'a> ExecutorContext<'a> {
         resp_tx: &UnboundedSender<ExecutorEvent<'a>>,
         req_rx: &mut UnboundedReceiver<RunUnitRequest<'a>>,
     ) -> Result<InternalExecuteStatus<'a>, ChildStartError> {
-        let ctx = TestExecuteContext {
-            profile_name: self.profile.name(),
-            double_spawn: &self.double_spawn,
-            target_runner: &self.target_runner,
-        };
+        let ctx = self.test_execute_context();
         let mut cmd = test.test_instance.make_command(
             &ctx,
             self.test_list,
@@ -679,12 +693,40 @@ impl<'a> ExecutorContext<'a> {
             &self.interceptor,
         );
 
+        let attempt_id = test.test_instance.id().attempt_id(
+            self.run_id,
+            test.stress_index.map(|s| s.current),
+            test.retry_data.attempt,
+        );
+
         let command_mut = cmd.command_mut();
 
-        // Debug environment variable for testing.
-        command_mut.env("__NEXTEST_ATTEMPT", format!("{}", test.retry_data.attempt));
-
+        // Test-related environment variables.
         command_mut.env("NEXTEST_RUN_ID", format!("{}", self.run_id));
+        command_mut.env(
+            "NEXTEST_BINARY_ID",
+            test.test_instance.suite_info.binary_id.as_str(),
+        );
+        command_mut.env("NEXTEST_TEST_NAME", test.test_instance.name);
+        command_mut.env("NEXTEST_ATTEMPT", test.retry_data.attempt.to_string());
+        command_mut.env(
+            "NEXTEST_TOTAL_ATTEMPTS",
+            test.retry_data.total_attempts.to_string(),
+        );
+        command_mut.env("NEXTEST_ATTEMPT_ID", &attempt_id);
+
+        let stress_current = test
+            .stress_index
+            .map(|s| s.current.to_string())
+            .unwrap_or_else(|| "none".to_owned());
+        let stress_total = match test.stress_index {
+            Some(stress_index) => stress_index
+                .total
+                .map_or_else(|| "unknown".to_owned(), |total| total.to_string()),
+            None => "none".to_owned(),
+        };
+        command_mut.env("NEXTEST_STRESS_CURRENT", stress_current);
+        command_mut.env("NEXTEST_STRESS_TOTAL", stress_total);
 
         // Set group and slot environment variables.
         command_mut.env(
@@ -784,11 +826,7 @@ impl<'a> ExecutorContext<'a> {
         let child_pid_for_kill = ChildPid::Process(child_pid);
 
         crate::fire_usdt!(UsdtTestAttemptStart {
-            attempt_id: test.test_instance.id().attempt_id(
-                self.run_id,
-                test.stress_index.map(|s| s.current),
-                test.retry_data.attempt,
-            ),
+            attempt_id,
             run_id: self.run_id,
             binary_id: test.test_instance.suite_info.binary_id.clone(),
             test_name: test.test_instance.name.to_owned(),
